@@ -88,4 +88,8 @@ Sparkline = (function(options){
   });
 
   $el.prepend($legend);
+});
+
+$(function(){
+  $('a[rel=tooltip]').tooltip();
 });
\ No newline at end of file
